# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding field 'KeyValue.updated'
        db.add_column('datatrans_keyvalue', 'updated',
                      self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now, auto_now=True, blank=True),
                      keep_default=False)

    def backwards(self, orm):
        # Deleting field 'KeyValue.updated'
        db.delete_column('datatrans_keyvalue', 'updated')


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'datatrans.fieldwordcount': {
            'Meta': {'unique_together': "(('content_type', 'field'),)", 'object_name': 'FieldWordCount'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'field': ('django.db.models.fields.CharField', [], {'max_length': '64', 'db_index': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'total_words': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'valid': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        },
        'datatrans.keyvalue': {
            'Meta': {'unique_together': "(('digest', 'language'),)", 'object_name': 'KeyValue'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'null': 'True'}),
            'digest': ('django.db.models.fields.CharField', [], {'max_length': '40', 'db_index': 'True'}),
            'edited': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'field': ('django.db.models.fields.TextField', [], {}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'default': 'None', 'null': 'True'}),
            'updated': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now', 'auto_now': 'True', 'blank': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'datatrans.modelwordcount': {
            'Meta': {'object_name': 'ModelWordCount'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']", 'unique': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'total_words': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'valid': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['datatrans']


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models

from datatrans.models import KeyValue
from collections import defaultdict

class Migration(DataMigration):

    depends_on = (
        ("datatrans", "0001_initial"),
    )

    def forwards(self, orm):
        "Write your forwards methods here."

        kv_map = defaultdict(lambda: [])

        for kv in orm.KeyValue.objects.all():
            key = (kv.language, kv.digest)
            kv_map[key].append(kv)

        for (language, digest), kv_list in kv_map.items():
            if len(kv_list) == 1:
                continue

            kv_list.sort(key=lambda kv: kv.id)

            for kv in kv_list[:-1]:
                print 'Deleting KeyValue', kv.id
                kv.delete()


    def backwards(self, orm):
        "Write your backwards methods here."

    models = {
        'datatrans.keyvalue': {
            'Meta': {'object_name': 'KeyValue'},
            'digest': ('django.db.models.fields.CharField', [], {'max_length': '40', 'db_index': 'True'}),
            'edited': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['datatrans']




from django.test import TestCase
from django.conf import settings
try:
    from django.db import DEFAULT_DB_ALIAS
except ImportError:
    pass
from django.utils import translation

from datatrans.models import KeyValue
from datatrans.utils import get_default_language

from test_project.testapp.models import Option
from test_project.testapp.utils import test_concurrently


if hasattr(settings, 'DATABASES'):
    DATABASE_ENGINE = settings.DATABASES[DEFAULT_DB_ALIAS]['ENGINE']
else:
    DATABASE_ENGINE = settings.DATABASE_ENGINE
USING_POSTGRESQL = DATABASE_ENGINE.startswith('postgresql') or \
    DATABASE_ENGINE.startswith('django.db.backends.postgresql')

class PostgresRegressionTest(TestCase):
    if USING_POSTGRESQL:
        def test_concurrent_inserts_with_same_value_break_pre_save(self):
            @test_concurrently(2)
            def add_new_records():
                value = "test string that does not already exist in db"
                option = Option(name=value)
                option.save()
                count_kv = KeyValue.objects.filter(language=get_default_language(),
                                                   value=value).count()
                self.assertEqual(count_kv, 1,
                                 u"Got %d KeyValues after concurrent insert instead of 1." % count_kv)
            add_new_records()

class RegressionTests(TestCase):
    def test_access_before_save_breaks_pre_save(self):
        translation.activate('en')
        value_en = "test1_en"
        option = Option(name=value_en)
        self.assertEqual(option.name, value_en)
        option.save()

        translation.activate('ro')
        self.assertEqual(option.name, value_en)
        value_ro = "test1_ro"
        option.name = value_ro
        self.assertEqual(option.name, value_ro)
        option.save()
        self.assertEqual(option.name, value_ro)

        translation.activate('en')
        value_en = "test2_en"
        option.name = value_en
        self.assertEqual(option.name, value_en) # this access causes the creation of a new KeyValue for 'en', which
                                                # causes the pre_save handler to skip creation of a new KeyValue for 'ro' language
                                                # and causes the last assertEqual to fail
        option.save()
        self.assertEqual(option.name, value_en)

        translation.activate('ro')
        self.assertEqual(option.name, value_ro)



VERSION = (0, 1, 4)
# 1.3 is currently on pypi


def get_version():
    if len(VERSION) > 3 and VERSION[3] != 'final':
        return '%s.%s.%s %s' % (VERSION[0], VERSION[1], VERSION[2], VERSION[3])
    else:
        return '%s.%s.%s' % (VERSION[0], VERSION[1], VERSION[2])


__version__ = get_version()


def autodiscover():
    """
    Same principle as for importing the admin modules with autodiscover() from django.contrib.admin
    """
    import copy
    import logging
    from django.conf import settings
    from django.contrib.admin.sites import site
    from django.utils.importlib import import_module
    from django.utils.module_loading import module_has_submodule

    logger = logging.getLogger(__name__)

    for app in settings.INSTALLED_APPS:
        mod = import_module(app)
        before_import_registry = copy.copy(site._registry)

        try:
            import_module('%s.datatranslation' % app)
            logger.info("Imported %s.datatranslation" % app)
        except:
            site._registry = before_import_registry

            if module_has_submodule(mod, 'datatranslation'):
               raise SystemError('Oh snap, weird stuff going on. We detected a submodule called \'datatrans\' in %s' % app)





# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'KeyValue'
        db.create_table('datatrans_keyvalue', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('digest', self.gf('django.db.models.fields.CharField')(max_length=40, db_index=True)),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=5, db_index=True)),
            ('value', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('edited', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('fuzzy', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('datatrans', ['KeyValue'])


    def backwards(self, orm):
        
        # Deleting model 'KeyValue'
        db.delete_table('datatrans_keyvalue')


    models = {
        'datatrans.keyvalue': {
            'Meta': {'object_name': 'KeyValue'},
            'digest': ('django.db.models.fields.CharField', [], {'max_length': '40', 'db_index': 'True'}),
            'edited': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['datatrans']



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    depends_on = (
        ("datatrans", "0001a_remove_duplicates"),
    )

    def forwards(self, orm):
        
        # Adding unique constraint on 'KeyValue', fields ['language', 'digest']
        db.create_unique('datatrans_keyvalue', ['language', 'digest'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'KeyValue', fields ['language', 'digest']
        db.delete_unique('datatrans_keyvalue', ['language', 'digest'])


    models = {
        'datatrans.keyvalue': {
            'Meta': {'unique_together': "(('digest', 'language'),)", 'object_name': 'KeyValue'},
            'digest': ('django.db.models.fields.CharField', [], {'max_length': '40', 'db_index': 'True'}),
            'edited': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['datatrans']



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import DataMigration
from django.db import models
from django.contrib.contenttypes.models import ContentType

from datatrans.models import make_digest, KeyValue
from datatrans.utils import get_registry
from collections import defaultdict

class Migration(DataMigration):

    def forwards(self, orm):
        "Write your forwards methods here."

        registry = get_registry()
        counts = defaultdict(lambda: [])

        for modelclass, fields in registry.items():
            ct = ContentType.objects.get_for_model(modelclass)
            for object in modelclass.objects.all():
                for field in fields.keys():
                    value = object.__dict__[field]
                    counts[value].append((object, field))
                    digest = make_digest(value)

                    done = {}

                    for kv in KeyValue.objects.filter(digest=digest).all():
                        if kv.object_id is None:
                            kv.content_object = object
                            kv.field = field
                            kv.save()
                        else:
                            if not kv.language in done:
                                KeyValue.objects.get_or_create(
                                    digest = kv.digest,
                                    language = kv.language,
                                    object_id = object.id,
                                    content_type_id = ct.id,
                                    field = field,
                                    defaults = { 'value': kv.value,
                                                 'edited': kv.edited,
                                                 'fuzzy': kv.fuzzy,
                                               }
                                )
                                done[kv.language] = 1

        # for value, uses in counts.items():
        #     if len(uses) > 1:
        #         print value, uses


    def backwards(self, orm):
        "Write your backwards methods here."


    models = {
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'datatrans.keyvalue': {
            'Meta': {'object_name': 'KeyValue'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'digest': ('django.db.models.fields.CharField', [], {'max_length': '40', 'db_index': 'True'}),
            'edited': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'field': ('django.db.models.fields.TextField', [], {}),
            'fuzzy': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5', 'db_index': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['datatrans']
